/*
 * Copyright (c) 1987, 1993
 *    The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in  the
 *    documentation and/or other materials provided with the  distribution.
 * 3. All advertising materials mentioning features or use of this  software
 *    must display the following acknowledgement:
 *    This product includes software developed by the University of
 *    California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its  contributors
 *    may be used to endorse or promote products derived from this  software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS''  AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR  PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE  LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR  CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,  STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY  OF
 * SUCH DAMAGE.
 */

#include <string.h>
#include <ctype.h>

#if defined(LIBC_SCCS) && !defined(lint)
static char sccsid[] = "@(#)strcasecmp.c      8.1 (Berkeley) 6/4/93";
#endif /* LIBC_SCCS and not lint */

#ifndef __STDC__
#define const
#endif

typedef unsigned char u_char;

int
strcasecmp(s1, s2)
      const char *s1, *s2;
{
      register const u_char
                      *us1 = (const u_char *)s1,
                      *us2 = (const u_char *)s2;

      while (tolower(*us1) == tolower(*us2++))
              if (*us1++ == '\0')
                      return (0);
      return (tolower(*us1) - tolower(*--us2));
}

int
strncasecmp(s1, s2, n)
      const char *s1, *s2;
      register size_t n;
{
      if (n != 0) {
              register const u_char
                              *us1 = (const u_char *)s1,
                              *us2 = (const u_char *)s2;

              do {
                      if (tolower(*us1) != tolower(*us2++))
                              return (tolower(*us1) - tolower(*--us2));
                      if (*us1++ == '\0')
                              break;
              } while (--n != 0);
      }
      return (0);
}

